#!/usr/bin/env python3
from mako.template import Template
import json, requests
from sys import exit
import os
import yaml
import pathlib
import argparse
from mako.lookup import TemplateLookup

def load_web_file(url):
  try:
    r = requests.get(url, allow_redirects=True, timeout=1)
  except:
    return url
  if r.status_code == 200:
    return r.text
  return url

def load_local_file(path,dir):
  try:
    f = pathlib.Path(pathlib.PurePath(dir,path))
    f_parents = f.parents[0]
    if f.is_file():
      return f.read_text(),f_parents
    else:
      print("Failed to read file: {}".format(str(f)))
      return "",f_parents
  except:
    return path,dir

def load_local_json(path,dir):
  try:
    text,new_dir = load_local_file(path,dir)
    o = json.loads(text)
    return o,new_dir
  except:
    return path,dir

def load_local_yaml(path,dir):
  try:
    text,new_dir = load_local_file(path,dir)
    o = yaml.safe_load(text)
    return o,new_dir
  except:
    return path,dir

#Returns object and whether it should be merged with the parent object
def check_obj(o,dir):
  if isinstance(o, str):
    if o[:1] == "<":
      merger = True
    elif o[:1] == "-":
      merger = False
    else:
      return o, False

    if o[:9] == "-https://" or o[:8] == "-http://":
      return load_web_file(o[1:],dir),merger
    elif o[:8] == "-file://":
      return load_local_file(o[8:],dir)[0],merger
    elif o[1:8] == "json://":
      new_o,new_dir = load_local_json(o[8:],dir)
      new_o = walk_obj(new_o,new_dir)
      return new_o,merger
    elif o[1:8] == "yaml://":
      new_o,new_dir = load_local_yaml(o[8:],dir)
      new_o = walk_obj(new_o,new_dir)
      return new_o,merger
    else:
      return o,merger
  else:
    return walk_obj(o,dir),False

def walk_obj(o,dir):
  if isinstance(o, list):
    new_obj = []
    for i in o:
      new_item,merger = check_obj(i,dir)
      if merger and isinstance(new_item,list):
        new_obj += new_item
      else:
        new_obj.append(new_item)
  elif isinstance(o, dict):
    new_obj = {}
    for k, v in o.items():
      new_item,merger = check_obj(v,dir)
      if merger and isinstance(new_item,dict):
        new_obj.update(new_item)
      else:
        new_obj[k] = new_item
  elif not isinstance(o,str):
    new_obj = o
  else:
    new_obj = check_obj(o,dir)[0]
  return new_obj


TEMPLATE_NAME="lp.template"
OUTPUT_NAME="example.template.adoc"
JSON_FILE="./example_17d_intro/17d_bolc/introduction/lesson1/lesson.json"
YAML_OUTPUT_FILE="./example_17d_intro/17d_bolc/introduction/lesson1/lesson.yaml"

parser = argparse.ArgumentParser(description="Lesson plan generator. Takes json object and uses it to build an asciidoctor document from a mako template. Optional arguments default to hardcoded values")

parser.add_argument("--lp-template", "-t", default=TEMPLATE_NAME, dest='template', help='path to mako template used to build final asciidoctor document.')
parser.add_argument("--output-file", "-o", default=OUTPUT_NAME, dest='output', help='path and name of output file for finale lesson plan.')
parser.add_argument("--input", "-i", default=JSON_FILE, dest='input', help='path json or yaml file used as input for mako template. Supported extensions include .yml, .yaml, and .json')
parser.add_argument("--convert", "-c", default=False, action="store_true", help='convert yaml to json or json to yaml')
parser.add_argument("--walk", "-w", default=False, action="store_true", help='when used with -c, will walk the the input object and resolve inclusions before converting. Ignored otherwise.')
parser.add_argument("--preview", "-p", default=False, action="store_true", help='preview final document to stdout before writing to disk')

args = parser.parse_args()

if not os.path.isfile(args.input):
    print("Error, input file {} does not exist".format(args.input))
    exit(1)
if not os.path.isfile(args.template):
    print("Error, template file {} does not exist".format(args.template))
    exit(1)

if args.input[-5:] == ".json":
  INPUT_TYPE = "json"
elif args.input[-5:] == ".yaml" or args.input[-4:] == ".yml":
  INPUT_TYPE = "yaml"
else:
  print("Error, unknown extension in {}.".format(args.input))
  exit(1)


mylookup = TemplateLookup(directories=["./"],input_encoding='utf-8', output_encoding='utf-8', encoding_errors='replace')

start_dir = pathlib.Path(args.input).parents[0]
print("start_dir is {}".format(start_dir))
f = open(args.input,"r")
try:
  key_values  = f.read()
  f.close()
  if INPUT_TYPE == "json":
      key_values = json.loads(key_values)
  else:
      key_values = yaml.load(key_values)

except Exception as e:
  print("Error loading input of type {}".format(INPUT_TYPE))
  print(e)
  exit(1)

if not args.convert or args.walk:
  key_values = walk_obj(key_values,start_dir)

if args.convert:
  if INPUT_TYPE == "json":
    f = open(args.input+".yaml","w")
    f.write(yaml.dump(key_values))
  elif INPUT_TYPE == "yaml":
    f = open(args.input+".json","w") 
    f.write(json.dumps(key_values))
  f.close()
  exit(0)



mytemplate = mylookup.get_template(args.template)
rendered = mytemplate.render(**key_values)

if args.preview:
  print(rendered)

f = open(args.output,"wb")
f.write(rendered)
f.close()
